/*****************************************************************************\
 *
 * vehicleActor.cpp
 *
 * VehicleActor - Class representing usable vehicle
 * VehicleActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <dtAudio/audiomanager.h>
#include <dtCore/scene.h>
#include <dtDAL/enginepropertytypes.h>
#include <dtDAL/functor.h>
#include <dtDAL/genericactorproperty.h>
#include <dtGame/basemessages.h>
#include <dtGame/defaultgroundclamper.h>
#include <dtGame/gameapplication.h>
#include <iostream>
#include "vehicleActor.h"
#include "MyMessages.h"

VehicleActor::VehicleActor(dtGame::GameActorProxy &proxy)
	: InteractiveActor(proxy)
	, isActive(false)
	, collision(false)
	, isLand(true)
	, isMoving(false)
	, oldPosition(0.f, 0.f, 0.f, 0.f, 0.f, 0.f)
	, newPosition(0.f, 0.f, 0.f, 0.f, 0.f, 0.f)
	, dust(NULL)
	, engineSound(NULL)
{
}

VehicleActor::~VehicleActor()
{
}

//"nizkourovnove" spravy
void VehicleActor::OnMessage(dtCore::Base::MessageData *data)
{
	if(isActive)
	{
		if(data->message == "collision")
		{
			dtCore::Scene::CollisionData* cd = static_cast<dtCore::Scene::CollisionData*>(data->userData);
		
			if((cd->mBodies[0]->GetName() == GetGameActorProxy().GetName()) || (cd->mBodies[1]->GetName() == GetGameActorProxy().GetName())) //kolidoval tento objekt
			{
				SetCollision(true);
			}
		}//collision
	}//active
}

//tick
void VehicleActor::OnTickLocal(const dtGame::TickMessage &tickMessage)
{
	if(isActive)
	{
		Update();
		if(isLand)
		{
			if(dust){
				if(!isMoving)
					dust->SetEnabled(true);
				else dust->ResetTime();
			}//dust
		}//land
	}//active
}

//spracovanie hernych sprav
void VehicleActor::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == MyMessageType::VEHICLE_ENTERED)
	{
		if(message.GetAboutActorId() == GetGameActorProxy().GetActor()->GetUniqueId())	//tento actor
		{
			SetIsActive(true);	//aktivny
			engineSound->Play();
		}
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_LEFT)
	{
		if(message.GetAboutActorId() == GetGameActorProxy().GetActor()->GetUniqueId())	//tento actor
		{
			SetIsActive(false);	//neaktivny
			engineSound->Stop();
		}
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_MOVES)
	{
		if(isLand)
		{
			isMoving = true;
		}
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_STOPPED)
	{
		if(isLand)
		{
			isMoving = false;
		}
	}
}

//inicializacia
void VehicleActor::Init()
{
	GetTransform(oldPosition);
	GetTransform(newPosition);
	GetTransform(initPosition);

	AddSender(&GetGameActorProxy().GetGameManager()->GetScene());

	if(isLand)
	{
		dust = new dtCore::ParticleSystem("dust");
		dust->LoadFile("Particles/prach.osg");
		dust->SetEnabled(false);
		dust->ResetTime();
		dust->SetTransform(dtCore::Transform(0.f, -1.f, -0.4f, 0, 0, 0), dtCore::Transformable::REL_CS);
		AddChild(dust.get());
	}

	SetSoundFile();
}

//je toto vozidlo aktivne?
bool VehicleActor::GetIsActive()
{
	return isActive;
}

//nastavime aktivny/neaktivny
void VehicleActor::SetIsActive(bool set)
{
	isActive = set;
}

//nastavi subor zvuk motora
void VehicleActor::SetSoundFile()
{
	engineSound = dtAudio::AudioManager::GetInstance().NewSound();
	engineSound->LoadFile("Sounds/helo.wav");
	engineSound->SetLooping(true);
	AddChild(engineSound.get());
}

//nastavime pozemny
void VehicleActor::SetIsLand(bool set)
{
	isLand = set;
}

bool VehicleActor::GetIsLand()
{
	return isLand;
}

//nastavenie priznaku o kolizii
void VehicleActor::SetCollision(bool set)
{
	collision = set;
}

//kolizie
void VehicleActor::Update()
{
	if(collision)
	{
		SetTransform(oldPosition);
		SetCollision(false);
	}else{
		oldPosition = newPosition;
		GetTransform(newPosition);
	}
}

/* ---------------------------------!!! VEHICLE ACTOR PROXY !!!-------------------------------------- */

VehicleActorProxy::VehicleActorProxy()
{
}

VehicleActorProxy::~VehicleActorProxy()
{
}

void VehicleActorProxy::BuildInvokables()
{
	InteractiveActorProxy::BuildInvokables();
}

void VehicleActorProxy::BuildPropertyMap()
{
	const std::string GROUP = "VehicleActor";

	VehicleActor &va = static_cast<VehicleActor&>(GetGameActor());

	InteractiveActorProxy::BuildPropertyMap();
	dtCore::RefPtr<dtDAL::BooleanActorProperty> prop = new dtDAL::BooleanActorProperty("isLand", 
												"isLand", 
												dtDAL::MakeFunctor(va, &VehicleActor::SetIsLand), 
												dtDAL::MakeFunctorRet(va, &VehicleActor::GetIsLand), 
												"Sets/gets whether actor is a land vehicle", 
												GROUP);
	AddProperty(prop.get());
}

void VehicleActorProxy::CreateActor()
{
	SetActor(*new VehicleActor(*this));
}

void VehicleActorProxy::OnEnteredWorld()
{
	//Registracia pre odber sprav od GM
	RegisterForMessages(MyMessageType::VEHICLE_ENTERED);
	RegisterForMessages(MyMessageType::VEHICLE_LEFT);
	RegisterForMessages(MyMessageType::VEHICLE_MOVES);
	RegisterForMessages(MyMessageType::VEHICLE_STOPPED);
	RegisterForMessages(dtGame::MessageType::TICK_LOCAL, dtGame::GameActorProxy::TICK_LOCAL_INVOKABLE);

	dtGame::GameActorProxy::OnEnteredWorld();
}